@@ -217,7 +217,7 @@ class Wolter_PH(guide_elements.GuideElement):
         #wolter.W = self.W
 
         end = self.current_instrument.add_component(self.end_component_name, "Arm")
-        end.set_AT([0, 0, self.get_length_name()], RELATIVE=wolter)
+        end.set_AT([0, 0, self.get_length_name()], RELATIVE=self.reference_component_name)
 
     def get_geometry(self):
         return GeometryWolter_PH(start_point=self.start_point_parameter,
@@ -266,14 +266,14 @@ class GeometryWolter_PH(BaseElementGeometry):
 
         if position < center - E_length:
             frac = position/(center - E_length)
-            return start_dim + frac*(self.rmax-start_dim)
+            return 2*(start_dim + frac*(self.rmax-start_dim))
 
         if position > center + H_length:
             frac = (position - center - H_length)/(length - center - H_length)
-            return self.rmax + frac*(end_dim - self.rmax)
+            return 2*(self.rmax + frac*(end_dim - self.rmax))
 
         if center - E_length <= position <= center + H_length:
-            return self.rmax
+            return 2*self.rmax
 
         raise RuntimeError("Failure in get_dimension logic for Wolter.")
 
